#include "asynckeygenerator.h"
#include "randomizer.h"

cAsyncKeyGenerator::cAsyncKeyGenerator( int keyLength, cAsyncKeyGenerator::sAlgorytm::eAlgorytm algo):
    recvKeys_(0)
{
    finalKey_ = new QByteArray( keyLength, '0' );
    keyLength_ = keyLength;
    algorytm_ = algo;
    //dispatchWorkers();
}

cAsyncKeyGenerator::~cAsyncKeyGenerator()
{
    foreach ( cAsyncGeneratorWorker * w, workers_)
    {
        delete w;
    }
}

void cAsyncKeyGenerator::dispatchWorkers()
{

    // Normal
    cAsyncGeneratorWorker * nworker = new cAsyncGeneratorWorker( keyLength_,
                                                                 cAsyncGeneratorWorker::sWorkerType::NORMAL );
    workers_.append( nworker );

    //Screen
    if( ( algorytm_ == sAlgorytm::SCREEN )|| ( algorytm_ == sAlgorytm::STRING ) )
    {
        cAsyncGeneratorWorker * scworker = new cAsyncGeneratorWorker( keyLength_,
                                                                      cAsyncGeneratorWorker::sWorkerType::SCREEN );
        workers_.append( scworker );
    }

    //String
    if( algorytm_ == sAlgorytm::STRING )
    {
        cAsyncGeneratorWorker * stworker = new cAsyncGeneratorWorker( keyLength_,
                                                                      cAsyncGeneratorWorker::sWorkerType::STRING );
        workers_.append( stworker );
    }

    foreach ( cAsyncGeneratorWorker * worker, workers_ )
    {
        connect( worker, SIGNAL(readyKey(QSharedPointer<QByteArray>,cAsyncGeneratorWorker::sWorkerType::eWorkerType) ),
                 this, SLOT(getArrayFromWorker(QSharedPointer<QByteArray>,cAsyncGeneratorWorker::sWorkerType::eWorkerType)) );
        connect( worker, SIGNAL(emitPercent(int,int)), this, SIGNAL(readyPercent(int,int)));
        worker->start();
        qDebug()<<"Start worker";
    }

}

void cAsyncKeyGenerator::getArrayFromWorker(QSharedPointer<QByteArray> arrPointer, cAsyncGeneratorWorker::sWorkerType::eWorkerType wtype)
{
    qDebug()<<"getWorkerKey";

    recvMutex_.lock();
    *finalKey_ = cFinalKeyGenerator::xOrByteArraysOfSameSize( arrPointer.data(), finalKey_  );
    recvKeys_++;
    recvMutex_.unlock();

    if( ( algorytm_ == sAlgorytm::NORMAL )&&( recvKeys_ == 1 ) )
    {

        emit readyKey( finalKey_ );
    }

    if( ( algorytm_ == sAlgorytm::SCREEN )&&( recvKeys_ == 2 ) )
    {

        emit readyKey( finalKey_ );
    }

    if( ( algorytm_ == sAlgorytm::STRING )&&( recvKeys_ == 3 ) )
    {

        emit readyKey( finalKey_ );
    }

    qDebug()<<"done emit key";
}


cAsyncGeneratorWorker::cAsyncGeneratorWorker( int keylength, cAsyncGeneratorWorker::sWorkerType::eWorkerType type)
{
    keyLength_ = keylength;
    type_ = type;

    if ( type == cAsyncGeneratorWorker::sWorkerType::STRING ){
        cRandomStringInputDialog dialog;
        dialog.setModal( true );
        QObject::connect( &dialog, SIGNAL ( sendStringGenerationData(QString, QList<QDateTime> ) ), this, SLOT( setStringAndDateTimes( QString , QList<QDateTime>  ) ) );
        dialog.exec();
    }
}

void cAsyncGeneratorWorker::setStringAndDateTimes(QString str, QList<QDateTime> list)
{
    str_=str;
    list_=list;
}

void cAsyncGeneratorWorker::run()
{

    cAbstractRandomizer * rnd = 0;


    if ( type_ == sWorkerType::NORMAL )
    {
        qDebug()<<"Normal worker";
        rnd = new cNormalRandomizer( keyLength_ );


    }
    else if ( type_ == sWorkerType::SCREEN )
    {
        qDebug()<<"Screen worker";
        rnd = new cScreenRandomizer( keyLength_ );
    }
    else
    {
        qDebug()<<"String worker";
        rnd = new cStringRandomizer( keyLength_, str_, list_ );
    }
    connect( rnd, SIGNAL(setProgress(int,int)), this, SIGNAL(emitPercent(int,int)) );

    rnd->gen();

    qDebug()<< "get key";
    QSharedPointer<QByteArray> arrPointer( rnd->getKey() );



    qDebug()<< "worker ready to emit key";
    emit readyKey( arrPointer,type_ );

}
