#ifndef ASYNCKEYGENERATOR_H
#define ASYNCKEYGENERATOR_H
#include <QSharedPointer>
#include <QByteArray>
#include <QThread>
#include <QMutex>
#include <QList>
#include <QDateTime>
#include <QString>

class cAsyncGeneratorWorker : public QThread
{
    Q_OBJECT

public:
    struct sWorkerType
    {
        enum eWorkerType
        {
            NORMAL,
            STRING,
            SCREEN
        };
    };


    cAsyncGeneratorWorker( int keylength,
                           sWorkerType::eWorkerType type);

    virtual void run();


private:
    cAsyncGeneratorWorker();

    int keyLength_;
    cAsyncGeneratorWorker::sWorkerType::eWorkerType type_;
    QString str_;
    QList<QDateTime> list_;

private slots:
void setStringAndDateTimes( QString str, QList<QDateTime>  list);


signals:
    void readyKey( QSharedPointer<QByteArray> arrPointer,
                   cAsyncGeneratorWorker::sWorkerType::eWorkerType wtype );
    void emitPercent( int percent, int worker );


};




class cAsyncKeyGenerator : public QObject
{
    Q_OBJECT

public:
    struct sAlgorytm
    {
        enum eAlgorytm
        {
            NORMAL,
            SCREEN,
            STRING
        };
    };

    cAsyncKeyGenerator( int keyLength,
                        cAsyncKeyGenerator::sAlgorytm::eAlgorytm algo );
    virtual ~cAsyncKeyGenerator();

    void dispatchWorkers();
private:
    cAsyncKeyGenerator(){}

    sAlgorytm::eAlgorytm algorytm_;
    QByteArray* finalKey_;
    int keyLength_;
    int recvKeys_;
    QList< cAsyncGeneratorWorker * > workers_;
    QMutex recvMutex_;

private slots:
    void getArrayFromWorker( QSharedPointer<QByteArray> arrPointer,
                             cAsyncGeneratorWorker::sWorkerType::eWorkerType wtype );

signals:
    void readyKey(  QByteArray* finalKey );
    void readyPercent( int maxpercent, int workerId );
};


#endif // ASYNCKEYGENERATOR_H
