#include <QFile>
#include <QMessageBox>
#include <QFileInfo>
#include "encryptdecrypt.h"
#include "randomizer.h"
#include "magicbytes.h"



cStreamEncryptor::cStreamEncryptor(const QString &keyFileName,
                                   const QString &message,
                                   const QString &saveFileName)
{
    QFile keyFile( keyFileName );

    if( ! keyFile.open( QIODevice::ReadWrite ) )
    {
        QMessageBox msgBox;
        msgBox.setText("Unable to open file");
        msgBox.exec();
        throw ("Unable to open file");
    }

    qDebug()<<"Prepare arrays";


    QDataStream keyFileHandler( &keyFile );

    sKey skey = extractKeyDataFromFileData( keyFileHandler );
    QByteArray encryptedData = encrypt( skey.key, message.toLocal8Bit(), skey.pos );
    saveFile( saveFileName, skey.pos, sStreamType::TEXT, encryptedData );
    keyFile.close();
    setNewKeyPosition( keyFileName, ( skey.pos + encryptedData.size() ), skey  );

}



cStreamEncryptor::cStreamEncryptor( const QString &keyFileName,
                                   QFile &encryptFile,
                                   const QString &saveFileName)
{
    qDebug()<<"FILE!";
    QFile keyFile( keyFileName );


    if( ! keyFile.open( QIODevice::ReadWrite ) )
    {
        QMessageBox msgBox;
        msgBox.setText("Unable to open file");
        msgBox.exec();
        throw ("Unable to open file");
    }

    //qDebug()<<"Prepare arrays";


    QDataStream keyFileHandler( &keyFile );

    QByteArray mainArray;
    encryptFile.open( QIODevice::ReadOnly );
    QDataStream parseStream( &mainArray, QIODevice::WriteOnly );
    qDebug()<< "ENCRYPTING FILE:"<<encryptFile.fileName();
    parseStream << QFileInfo ( encryptFile.fileName() ).fileName() << encryptFile.readAll();
    encryptFile.close();





    sKey skey = extractKeyDataFromFileData( keyFileHandler );
    QByteArray encryptedData = encrypt( skey.key, mainArray, skey.pos );
    saveFile( saveFileName, skey.pos, sStreamType::FILE, encryptedData );
    keyFile.close();
    setNewKeyPosition( keyFileName, ( skey.pos + encryptedData.size() ), skey  );

}



const QByteArray cStreamEncryptor::encrypt(const QByteArray key,
                                           const QByteArray target,
                                           int keyPosition )
{

    QByteArray portion;
    if ( key.size() < keyPosition  ) throw ( "Key too small or key is depleted" );
    if ( ( key.size() - target.size() ) < keyPosition  ) throw ( "Key too small or key is depleted" );
    for ( int i=0; i<target.size(); i++ )
    {
        portion.append( key.at( keyPosition + i ) );
    }

    return cFinalKeyGenerator::xOrByteArraysOfSameSize( &portion, &target );

}

cStreamEncryptor::sKey cStreamEncryptor::extractKeyDataFromFileData( QDataStream &fileDataStream )
{
    cStreamEncryptor::sKey keyStruct;

    qint64 magicByte;
    fileDataStream >> magicByte >> keyStruct.length >> keyStruct.pos >> keyStruct.key;




    qDebug() << "KEYDATA" << magicByte << keyStruct.length << keyStruct.pos;// << keyStruct.key;
    if ( magicByte != KEYSIG ) throw "bad keyfile";
    return keyStruct;
}

bool cStreamEncryptor::setNewKeyPosition(const QString &keyFileName, qint64 newPos, const cStreamEncryptor::sKey keyStruct)
{


    QFile senderFile( keyFileName );
    senderFile.open( QIODevice::WriteOnly );
    QDataStream senderOut( &senderFile );
    qint64 mbyte = KEYSIG;
    senderOut<<mbyte;
    qint64 len = keyStruct.length;
    senderOut<<len;//length
    qint64 pos = newPos;
    senderOut<<pos;//begin
    senderOut<<keyStruct.key;
    //senderOut.flush();
    senderFile.close();


    return true;
}



void cStreamEncryptor::saveFile( QString fileName, int keyPosition, cStreamEncryptor::sStreamType::eStreamType type, const QByteArray &data )
{
    qDebug()<<fileName;
    QFile outFile( fileName );
    if( ! outFile.open( QIODevice::WriteOnly ) )
    {
        throw "can not open output file";
    }

    QDataStream out( &outFile );
    qint64 mbytes;

    if( type == cStreamEncryptor::sStreamType::TEXT )
    {
        mbytes = TEXTSIG;
    }
    else
    {
        mbytes = FILESIG;
    }
qDebug() << "MBYTES" << mbytes;
    qint64 keyPos = keyPosition;

    out<<mbytes<<keyPos<<data;
    outFile.close();


}
