#include "encryptdialog.h"
#include "ui_encryptdialog.h"
#include "encryptdecrypt.h"
#include <QMessageBox>
#include <QFileDialog>

cEncryptDialog::cEncryptDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::cEncryptDialog)
{
    ui->setupUi(this);
    ui->messagePlainTextEdit->setEnabled( true );
    ui->fileSelectGroupBox->setEnabled( false );
}

cEncryptDialog::~cEncryptDialog()
{
    delete ui;
}

void cEncryptDialog::on_messageModeRadioButton_toggled(bool checked)
{
    ui->messagePlainTextEdit->setEnabled( checked );
    ui->fileSelectGroupBox->setEnabled( ! checked );
}


void cEncryptDialog::on_fileModeRadioButton_toggled(bool checked)
{
    ui->fileSelectGroupBox->setEnabled( checked );
    ui->messagePlainTextEdit->setEnabled( ! checked );
}

void cEncryptDialog::on_buttonBox_accepted()
{
    // TODO check keyfile exists
    if ( ui->KeyFileNameLineEdit->text() == QString() )
    {
        QMessageBox msgBox;
        msgBox.setText("No keyfile given");
        msgBox.exec();
        return;
    }

    if ( ui->OutputFileNameLineEdit->text() == QString() )
    {
        QMessageBox msgBox;
        msgBox.setText("No output file name given");
        msgBox.exec();
        return;
    }

    try
    {
        if ( ui->messageModeRadioButton->isChecked() )
        {
            cStreamEncryptor encryptor( ui->KeyFileNameLineEdit->text(), ui->messagePlainTextEdit->toPlainText(), ui->OutputFileNameLineEdit->text() );
        }
        else
        {
            QFile encryptFile( ui->fileNameLineEdit->text() );
            if ( ! encryptFile.open( QIODevice::ReadOnly ) )
            {
                QMessageBox msgBox;
                msgBox.setText("Can not open target file to encrypt");
                msgBox.exec();
                this->show();
                return;
            }
            encryptFile.close();
            cStreamEncryptor encryptor (  ui->KeyFileNameLineEdit->text(), encryptFile, ui->OutputFileNameLineEdit->text() );
        }
    }
    catch (const char * err)
    {
        QMessageBox msgBox;
        msgBox.setText( err );
        msgBox.exec();
        return;
    }
}

void cEncryptDialog::on_keyFileSelectPushButton_clicked()
{
    QString fileName = QFileDialog::getOpenFileName(this, tr("Open Sender Encryption File"),
                                                    "",
                                                    tr("Sender key files (*.key)"));
    ui->KeyFileNameLineEdit->setText( fileName );
    /// enk - enkcryption (sender key). dnk - decryption (reciever key)
}

void cEncryptDialog::on_OutputFileSelectPushButton_clicked()
{
    QString filter = tr("Encrypted container (*.ems)");
    QString saveName = QFileDialog::getSaveFileName(this,  tr("Save as"), "", filter, &filter );
    QFileInfo f( saveName );
    if (f.suffix().isEmpty())
    {
        saveName+=".ems";
    }

    ui->OutputFileNameLineEdit->setText( saveName );
}

void cEncryptDialog::on_fileSelectPushButton_clicked()
{
    QString fileName = QFileDialog::getOpenFileName(this, tr("Encrypted File"),
                                                    "",
                                                    tr("All files (*.*)"));
    ui->fileNameLineEdit->setText( fileName );
}
