#include "randomizer.h"
#include "newkeygenerationdialog.h"


#include <QDebug>
#include <QChar>
#include <QScreen>
#include <QDir>
#include <QBuffer>
#include <QDateTime>
#include <QMessageBox>


int cAbstractRandomizer::id_ = 0;

cNormalRandomizer::~cNormalRandomizer()
{

}

cNormalRandomizer::cNormalRandomizer( int keyLength )
{
    qsrand( QDateTime::currentDateTime().toTime_t() );
    keyLength_ = keyLength;

}

void cNormalRandomizer::gen()
{
    emit setProgress( 0 ,cId_);
    //meter_->setName( "NORMAL RANDOMIZER" );
    //meter_->show();
    key_ = new QByteArray();
    for (int i=0; i<keyLength_; i++)
    {
        //qDebug()<<"i"<<i<<"Kl"<<keyLength<<(i*100/keyLength);

        char s = qrand()%256 ;
        key_->push_back( s );

    }
    //meter_->hide();
    //   qDebug()<<*key_;
    emit setProgress( 100 ,cId_);
}

int cAbstractRandomizer::convertToPc(double x, double base)
{
    double bx = x/base;
    bx = bx * 100;
    int a = bx;
    return a;
}

cAbstractRandomizer::cAbstractRandomizer()
{
    keyLength_ = 0;
    cId_ = cAbstractRandomizer::id_;
    cAbstractRandomizer::id_++;
  // meter_ = new cProgressMeter( 0 );
  // QObject::connect( this, SIGNAL( setProgress(int) ), meter_, SLOT(setProgress(int)) );
}

QByteArray *cAbstractRandomizer::getKey()
{
    return key_;
}


cScreenRandomizer::~cScreenRandomizer()
{

}

cScreenRandomizer::cScreenRandomizer( int keyLength )
{
    qsrand( QDateTime::currentDateTime().toTime_t() );
    keyLength_=keyLength;

}

void cScreenRandomizer::gen()
{
    key_ = new QByteArray();
    /// Get image and turn it in array of bytes
    ///
    emit setProgress( 0 ,cId_);
    QPixmap screen = getScreen();

    QImage image = screen.toImage();

    QByteArray imageKey;//( reinterpret_cast<const char*>( image.constBits() ) );
    QBuffer buffer(&imageKey);
    buffer.open(QIODevice::WriteOnly);
    image.save(&buffer, "PNG");


    /// Apply random mask to image key
    QByteArray randomizedImageKey;
    //meter_->setName( "SCREEN RANDOMIZER - step 1 of 2" );
    //meter_->show();
    for ( int i=0; i<imageKey.size(); i++ )
    {

        char s ( qrand()%256 );
        char rd = imageKey[i];
        char r = (char)(rd ^ s);

        randomizedImageKey.push_back( r );
    }


    /// Take random numbers from randomized image key into key array
    //meter_->setName( "SCREEN RANDOMIZER - step 2 of 2" );
    for ( int i=0; i < keyLength_ ; i++ )
    {

        int m = qrand()%(randomizedImageKey.size());
        char ins = randomizedImageKey[ m ];

        key_->push_back( ins );

    }
    emit setProgress( 100 ,cId_);
    //meter_->hide();
}

QPixmap cScreenRandomizer::getScreen()
{
    QScreen *screen = QGuiApplication::primaryScreen();
    if (! screen) throw "No screen captured";

    return screen->grabWindow(0);


}


cStringRandomizer::cStringRandomizer(int keyLength, QString string, QList<QDateTime> dateTimes)
{
    qsrand( QDateTime::currentDateTime().toTime_t() );
    tempLength_ = keyLength;
    key_ = new QByteArray();
    setStringAndDateTimes( string, dateTimes );



}

cStringRandomizer::~cStringRandomizer()
{

}

void cStringRandomizer::gen()
{
     startGen();
}

cStringRandomizer::cStringRandomizer( int keyLength )
{
    qsrand( QDateTime::currentDateTime().toTime_t() );
    keyLength_ = keyLength;
    tempLength_ = keyLength_;
    key_ = new QByteArray();
    cRandomStringInputDialog dialog;
    QObject::connect( &dialog, SIGNAL ( sendStringGenerationData(QString, QList<QDateTime> ) ), this, SLOT( setStringAndDateTimes( QString , QList<QDateTime>  ) ) );
    dialog.exec();
}

void cStringRandomizer::setStringAndDateTimes(QString string, QList<QDateTime> dateTimes)
{
    //meter_->setName( "STRING RANDOMIZER" );
    //meter_->show();
    // qDebug()<<"HURRAY!"<<string<<dateTimes<<dateTimes[1].toMSecsSinceEpoch()<<dateTimes[2].toMSecsSinceEpoch();
    dTimes_ = datesToBytes( dateTimes );
    bStr_ = stringToBytes( string );


}
void cStringRandomizer::startGen()
{
    emit setProgress( 0, cId_ );
    for ( int i=0; i<tempLength_; i++ )
    {

        char c = dTimes_ [ qrand()%( dTimes_.size() - 1 ) ];
        char d = bStr_ [ qrand()%( bStr_.size() - 1 ) ];
        char e = static_cast< char > (c ^ d);
        key_->push_back( e );
    }

    emit setProgress( 100, cId_ );
    //meter_->hide();

}

QByteArray cStringRandomizer::stringToBytes(QString string)
{
    QByteArray ba = string.toUtf8();

    QByteArray fBa;

    for( int i=0; i<ba.size(); i++ )
    {
        char c = ba[i];
        char d = static_cast< char >( qrand()%256 );
        char cd = static_cast< char > (c ^ d);
        fBa.append( cd );
    }

    return fBa;
}

QByteArray cStringRandomizer::datesToBytes( QList<QDateTime> dateTimes )
{
    QByteArray ba;
    foreach ( QDateTime dateTime , dateTimes ) {
        ba.append( dateTime.toString().toUtf8() );
    }

    QByteArray fBa;

    for( int i=0; i<ba.size(); i++ )
    {
        char c = ba[i];
        char d = static_cast< char >( qrand()%256 );
        char cd = static_cast< char > (c ^ d);
        fBa.append( cd );
    }

    return fBa;
}




cFinalKeyGenerator::cFinalKeyGenerator()
{

    qsrand( QDateTime::currentDateTime().toTime_t() );

    meter_ = new cProgressMeter(0);
    meter_->show();
    if ( cFinalKeyGenerator::registered_ == false ) {
        cFinalKeyGenerator::registered_ = true;
        qRegisterMetaType< QSharedPointer<QByteArray> >("QSharedPointer<QByteArray>");
        qRegisterMetaType< cAsyncGeneratorWorker::sWorkerType::eWorkerType >("cAsyncGeneratorWorker::sWorkerType::eWorkerType");
    }
}

bool cFinalKeyGenerator::registered_ = false;

cFinalKeyGenerator::~cFinalKeyGenerator()
{
    qDebug()<<"~cFinalKeyGenerator";
    gen_->deleteLater();
    meter_->deleteLater();
}

void cFinalKeyGenerator::gen()
{
    cNewKeyGenerationDialog dialog;
    QObject::connect( &dialog, SIGNAL(sendInnerData(int,QString, int)), this, SLOT(setInnerDataFromDialog(int,QString, int )) );
    dialog.exec();

}

QByteArray cFinalKeyGenerator::xOrByteArraysOfSameSize(const QByteArray *one, const QByteArray *two )
{
    if ( one->size() != two->size() ) throw QString("Array size MisMatch");
    QByteArray finArray;
    for (int i=0; i<one->size(); i++)
    {
        char c = one->at(i);
        char d = two->at(i);
        char cd =  static_cast< char > (c ^ d);
        finArray.append( cd );
    }
    return finArray;
}

void cFinalKeyGenerator::setInnerDataFromDialog(int keyLength, QString keyDir, int genType)
{
    keyLength_ = keyLength;
    keyDir_ = keyDir;
    genType_ = static_cast<cFinalKeyGenerator::sGenType::eGenType> (genType);

    startGeneration();
}

void cFinalKeyGenerator::showWorkerInfo(int maxpercent, int workerId)
{
    while ( workerId > ( maxProgress_.size() - 1 ) )
    {
        maxProgress_.append(-1);
    }
    if ( maxProgress_[workerId]  == -1 )
    {
        maxProgress_[workerId] = 0;
        QString am = meter_->text();
        am += "Worker started: "+QString::number( workerId ) + "\n";
        meter_->setText( am );
    }

    if( maxProgress_[workerId] == maxpercent ) return;
    maxProgress_[workerId] = maxpercent;

    if( maxpercent == 100 )
    {
        QString am = meter_->text();
        am += "Worker finished: "+QString::number( workerId ) + "\n";
        meter_->setText( am );
    }



}

void cFinalKeyGenerator::startGeneration()
{
    // QByteArray * finalKey = new QByteArray() ;
    qDebug()<<"START GENERATION";
    switch (genType_) {
    case  cFinalKeyGenerator::sGenType::FAST:
    {
        qDebug()<<"FASTGEN";

        gen_= new cAsyncKeyGenerator( keyLength_,
                                      cAsyncKeyGenerator::sAlgorytm::NORMAL );
        //        cNormalRandomizer randGen( keyLength_ );
        //        *finalKey = *(randGen.getKey());
        break;
    }
    case cFinalKeyGenerator::sGenType::NORMAL:
    {
        qDebug()<<"NORMGEN";
        gen_= new cAsyncKeyGenerator( keyLength_,
                                      cAsyncKeyGenerator::sAlgorytm::SCREEN );
        //        cNormalRandomizer randGen( keyLength_ );
        //        cScreenRandomizer sRandGen( keyLength_ );
        /* cRandomContainerThreadNormalRandomizer randGen( keyLength_ );
        cRandomContainerThreadScreenRandomizer sRandGen( keyLength_ );
        randGen.start();
        sRandGen.start();
        randGen.wait();
        sRandGen.wait();*/
        //        meter_->setName( "Final key generation - it may take a while" );

        //        meter_->setProgressState( false );
        //        meter_->show();
        //        meter_->setProgress( 50 );

        //        *finalKey = xOrByteArraysOfSameSize( randGen.getKey(), sRandGen.getKey() );
        //        meter_->hide();
        break;
    }
    case cFinalKeyGenerator::sGenType::PARANOIC:
    {

        qDebug()<<"PARAGEN";
        gen_= new cAsyncKeyGenerator( keyLength_,
                                      cAsyncKeyGenerator::sAlgorytm::STRING );
        //        cNormalRandomizer randGen( keyLength_ );
        //        cScreenRandomizer sRandGen( keyLength_ );
        //        cStringRandomizer stRandGen( keyLength_ );
        //        meter_->setName( "Final key generation - it may take a while" );

        //        meter_->setProgressState( false );
        //        meter_->show();
        //        meter_->setProgress( 50 );

        /*cRandomContainerThreadNormalRandomizer randGen( keyLength_ );
        cRandomContainerThreadScreenRandomizer sRandGen( keyLength_ );
        cRandomContainerThreadStringRandomizer stRandGen( keyLength_ );
        randGen.start();
        sRandGen.start();
        stRandGen.start();

        randGen.wait();
        sRandGen.wait();
        stRandGen.wait();*/
        //        *finalKey = xOrByteArraysOfSameSize( randGen.getKey(), sRandGen.getKey() );
        //        *finalKey = xOrByteArraysOfSameSize( finalKey, stRandGen.getKey() );
        //        meter_->hide();
        break;
    }
    default:
        throw "NOT IMPLEMENTED";
        break;
    }

    connect( gen_, SIGNAL( readyKey(QByteArray*) ), this, SLOT(endGeneration(QByteArray*)) );
    connect( gen_, SIGNAL(readyPercent(int,int)), this, SLOT(showWorkerInfo(int,int)) );

    gen_->dispatchWorkers();

}

void cFinalKeyGenerator::endGeneration( QByteArray *finalKey )
{
    meter_->setText("Generating Final Key");
    qDebug()<<"FINAL KEY:"<<finalKey->size();//<<*finalKey;
    // QINT64 solely for further developement
    QString senderFilePath = keyDir_+"/sender.key";
    QString receiverFilePath = keyDir_+"/reciever.rkey";

    QFile senderFile( senderFilePath );
    QFile receiverFile( receiverFilePath );

    if(( ! senderFile.open( QIODevice::WriteOnly ) ) ||
            ( ! receiverFile.open( QIODevice::WriteOnly ) ) )
    {
        throw "can not open files";
    }

    // FIXME: Fails at long keys

    qDebug()<<"KEYLENGTHS"<<keyLength_<<finalKey->size();

    QDataStream senderOut(&senderFile);
    qint64 mbyte = 777;
    senderOut<<mbyte;
    qint64 len = keyLength_;
    senderOut<<len;//length
    qint64 pos = 0;
    senderOut<<pos;//begin
    senderOut<<*finalKey;
    //senderOut.flush();
    senderFile.close();

    QDataStream receiverOut(&receiverFile);
    receiverOut<<*finalKey;
    // receiverOut.flush();
    receiverFile.close();
    meter_->hide();
    delete finalKey;



QMessageBox::information(  0, "Key file generation done",

                           QString(QObject::tr("Generated\n") )+
                                   senderFilePath+"\n"+
                                   receiverFilePath,
                           QMessageBox::Ok);


}
