/*
   QT Vernam, a program to implement Vernam algorythm for very strong file
   and message encryption.
   Copyright (C) 2014  Vokhmin Ivan <i_vohmin@mail.ru>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>
*/
#ifndef RANDOMIZER_H
#define RANDOMIZER_H

#include <QThread>
#include <QByteArray>
#include <QPixmap>
#include <QApplication>
#include <QDesktopWidget>
#include <QDebug>
#include <QList>
#include <QDateTime>
#include <QAbstractButton>
#include "randomstringinputdialog.h"
#include "progressmeter.h"
#include "asynckeygenerator.h"

class cAbstractRandomizer : public QObject
{

Q_OBJECT

public:
    static int convertToPc( double x, double base );


   cAbstractRandomizer();
    virtual ~cAbstractRandomizer(){}

    virtual void gen()=0;
    virtual QByteArray * getKey();
signals:
    void setProgress( int percent, int meterId );

protected:

    QByteArray * key_;
    int keyLength_;
    static int id_;
    int cId_;
   // cProgressMeter * meter_;
};

class cNormalRandomizer : public cAbstractRandomizer
{
public:

    virtual ~cNormalRandomizer();
    cNormalRandomizer(int keyLength );
    virtual void gen();

private:
    cNormalRandomizer();
};



class cScreenRandomizer : public cAbstractRandomizer
{
public:
    virtual ~cScreenRandomizer();
    cScreenRandomizer( int keyLength );
    virtual void gen();

private:
    cScreenRandomizer();

    QPixmap getScreen();
};



class cStringRandomizer : public cAbstractRandomizer
{
    Q_OBJECT

public:
    cStringRandomizer( int keyLength );
    cStringRandomizer( int keyLength, QString string, QList<QDateTime> dateTimes );

   virtual ~cStringRandomizer();

    virtual void gen();
    virtual QByteArray * getKey() { return key_; }

    void startGen();

public slots:

   void setStringAndDateTimes(QString string, QList<QDateTime> dateTimes );

private:
   QByteArray dTimes_;
   QByteArray bStr_;
   QByteArray stringToBytes( QString string );
   QByteArray datesToBytes(  QList<QDateTime> dateTimes );

   int tempLength_;

};



class cFinalKeyGenerator : public cAbstractRandomizer
{

    Q_OBJECT

public:
    struct sGenType
    {
        enum eGenType
        {
            FAST,
            NORMAL,
            PARANOIC
        };
    };

    cFinalKeyGenerator();
   virtual ~cFinalKeyGenerator();

    virtual void gen();
    static QByteArray xOrByteArraysOfSameSize(const QByteArray *one, const QByteArray *two );


public slots:
    void setInnerDataFromDialog( int keyLength, QString keyDir, int genType );
    void showWorkerInfo( int maxpercent, int workerId);

private:

    void startGeneration();



    int keyLength_;
    QString keyDir_;
    sGenType::eGenType genType_;
    cAsyncKeyGenerator * gen_;
    static bool registered_;
    cProgressMeter * meter_;
    QList<int> maxProgress_;


 private slots:
     void endGeneration( QByteArray * array );

};



#endif // RANDOMIZER_H
